import json
import os
from typing import List, Optional, Dict

from pydantic import BaseModel


class Steps(BaseModel):
    """ Information about a specific step of GTDB-Tk"""
    name: str
    output_dir: Optional[str]
    starts_at: Optional[str]
    ends_at: Optional[str]
    duration: Optional[str]
    status: Optional[str]

    def is_complete(self) -> bool:
        if self.status == "completed":
            return True
        else:
            return False

class ANIScreenStep(Steps):
    """ Information about the ANI screen step of GTDB-Tk"""
    name: str = "ANI screen"
    genome_dir: Optional[str]
    batchfile: Optional[str]
    mash_k: Optional[str]
    mash_s: Optional[str]
    mash_v: Optional[str]
    mash_max_dist: Optional[str]
    mash_db: Optional[str]
    output_files: Optional[Dict]

class IdentifyStep(Steps):
    """ Information about the Identify step of GTDB-Tk"""
    name: str = "identify"
    genes: Optional[bool]
    extension: Optional[str]
    write_single_copy_genes: Optional[bool]
    genome_dir: Optional[str]
    batchfile: Optional[str]
    output_files: Optional[Dict]

class AlignStep(Steps):
    """ Information about the Align step of GTDB-Tk"""
    name: str = "align"
    identify_dir: Optional[str]
    skip_gtdb_refs: Optional[bool]
    taxa_filter: Optional[str]
    min_perc_aa: Optional[str]
    custom_msa_filters: Optional[bool]
    skip_trimming: Optional[bool]
    rnd_seed: Optional[str]
    cols_per_gene: Optional[str]
    min_consensus: Optional[str]
    max_consensus: Optional[str]
    min_perc_taxa: Optional[str]
    outgroup_taxon: Optional[str]
    output_files: Optional[Dict]

class ClassifyStep(Steps):
    """ Information about the Classify step of GTDB-Tk"""
    name: str = "classify"
    align_dir: Optional[str]
    genome_dir: Optional[str]
    batchfile: Optional[str]
    scratch_dir: Optional[str]
    debug_option: Optional[bool]
    full_tree: Optional[bool]
    skip_ani_screen: Optional[bool]
    no_mash: Optional[bool]
    mash_k: Optional[str]
    mash_v: Optional[str]
    mash_s: Optional[str]
    mash_db: Optional[str]
    mash_max_dist: Optional[str]
    output_files: Optional[Dict]

class InferStep(Steps):
    """ Information about the Infer step of GTDB-Tk"""
    name: str = "infer"
    msa_file: Optional[str]
    prot_model: Optional[str]
    no_support: Optional[bool]
    gamma: Optional[bool]

class RootStep(Steps):
    """ Information about the Root step of GTDB-Tk"""
    name: str = "root"
    input_tree: Optional[str]
    gtdbtk_classification_file: Optional[str]
    custom_taxonomy_file: Optional[str]
    outgroup_taxon: Optional[str]

class DecorateStep(Steps):
    """ Information about the Decorate step of GTDB-Tk"""
    name: str = "decorate"
    input_tree: Optional[str]
    gtdbtk_classification_file: Optional[str]
    custom_taxonomy_file: Optional[str]
    suffix: Optional[str]
    output_files: Optional[Dict]



class StageLogger(object):
    class __StageLogger(BaseModel):

        version: Optional[str]
        command_line: Optional[str]
        database_version: Optional[str]
        database_path: Optional[str]
        steps: List[Steps]
        output_dir: Optional[str]
        path: Optional[str]

        def __str__(self):
            return repr(self) + self.val

        def write(self):
            with open(self.path, "w") as f:
                f.write(self.json(indent=4))

        def has_stage(self, stage_object: object) -> bool:
            processed_steps = [x for x in self.steps if isinstance(x, stage_object)]
            if len(processed_steps) > 0:
                return True
            else:
                return False

        def get_stage(self, stage_object: object) -> object:
            processed_steps = [x for x in self.steps if isinstance(x, stage_object)]
            if len(processed_steps) > 0:
                return processed_steps[0]
            else:
                return None

        # create reset steps function, which takes an optional list of steps to keep
        # if no list is provided, all steps are removed

        def reset_steps(self, keep_steps: List[str] = None):
            if keep_steps is None:
                self.steps = []
            else:
                self.steps = [x for x in self.steps if x.name in keep_steps]

        def read_existing_steps(self):
            with open(self.path, "r") as f:
                data = json.load(f)
                #stage_logger = self.parse_obj(data)
                steps = []
                for step in data.get('steps'):
                    if step.get('name') == "ANI screen":
                        step_object = ANIScreenStep(**step)
                    elif step.get('name') == "identify":
                        step_object = IdentifyStep(**step)
                    elif step.get('name') == "align":
                        step_object = AlignStep(**step)
                    elif step.get('name') == "classify":
                        step_object = ClassifyStep(**step)
                    elif step.get('name') == "infer":
                        step_object = InferStep(**step)
                    else:
                        raise Exception(f"Unknown step name {step.name}")
                    steps.append(step_object)
                self.steps = steps

    instance = None
    def __new__(cls): # __new__ always a classmethod
        if not StageLogger.instance:
            StageLogger.instance = StageLogger.__StageLogger(steps=[])
        return StageLogger.instance
    def __getattr__(self, name):
        return getattr(self.instance, name)
    def __setattr__(self, name):
        return setattr(self.instance, name)





# class StageLoggerFile:
#     def __init__(self,output_dir: str):
#         self.output_dir = output_dir
#
#     # We generate the StageLogger object
#     def setStageLogger(self, version:str, command_line: str,
#                  database_version: str, database_path: str):
#         self.path = os.path.join(self.output_dir, "gtdbtk.json")
#         self.stage_logger = StageLogger(version = version,
#                                         command_line=command_line,
#                                         database_version=database_version,
#                                         database_path=database_path,
#                                         steps=[])
#     def write(self):
#         with open(self.path, "w") as f:
#             f.write(self.stage_logger.json(indent=4))
#
#     def read(self):
#         with open(self.path, "r") as f:
#             self.stage_logger = StageLogger.parse_raw(f.read())
#             steps = []
#             for step in self.stage_logger.steps:
#                 if step.name == "ANI screen":
#                     step_object = ANIScreenStep(**step.dict())
#                 elif step.name == "identify":
#                     step_object = IdentifyStep(**step.dict())
#                 elif step.name == "align":
#                     step_object = AlignStep(**step.dict())
#                 elif step.name == "classify":
#                     step_object = ClassifyStep(**step.dict())
#                 elif step.name == "infer":
#                     step_object = InferStep(**step.dict())
#                 else:
#                     raise Exception(f"Unknown step name {step.name}")
#                 steps.append(step_object)
#             self.stage_logger.steps = steps


import json
import os
import sys
from functools import lru_cache


class __GTDBTkCommonConfig:
    """
    This class encapsulates all configuration options. It will protect against
    importing code that depends on a specific value throwing an exception
    that requires the setting of the GTDB-Tk reference data path.
    """

    MIN_REF_DATA_VERSION = 'r207'
    COMPATIBLE_REF_DATA_VERSIONS = ['r207','r214']

    BACKBONE_PPLACER_REF_PKG = 'gtdbtk_package_backbone.refpkg'
    CLASS_LEVEL_PPLACER_REF_PKG = 'gtdbtk.package.{iter}.refpkg'

    # Relative Evolution Distance
    RED_INTERVAL = 0.1
    RED_MIN_SUPPORT = 0.0
    RED_MIN_CHILDREN = 2

    # Marker information
    BAC120_MARKERS = {"PFAM": ["PF00380.20.hmm", "PF00410.20.hmm", "PF00466.21.hmm",
                               "PF01025.20.hmm", "PF02576.18.hmm", "PF03726.15.hmm"],
                      "TIGRFAM": ["TIGR00006.HMM", "TIGR00019.HMM", "TIGR00020.HMM",
                                  "TIGR00029.HMM", "TIGR00043.HMM", "TIGR00054.HMM",
                                  "TIGR00059.HMM", "TIGR00061.HMM", "TIGR00064.HMM",
                                  "TIGR00065.HMM", "TIGR00082.HMM", "TIGR00083.HMM",
                                  "TIGR00084.HMM", "TIGR00086.HMM", "TIGR00088.HMM",
                                  "TIGR00090.HMM", "TIGR00092.HMM", "TIGR00095.HMM",
                                  "TIGR00115.HMM", "TIGR00116.HMM", "TIGR00138.HMM",
                                  "TIGR00158.HMM", "TIGR00166.HMM", "TIGR00168.HMM",
                                  "TIGR00186.HMM", "TIGR00194.HMM", "TIGR00250.HMM",
                                  "TIGR00337.HMM", "TIGR00344.HMM", "TIGR00362.HMM",
                                  "TIGR00382.HMM", "TIGR00392.HMM", "TIGR00396.HMM",
                                  "TIGR00398.HMM", "TIGR00414.HMM", "TIGR00416.HMM",
                                  "TIGR00420.HMM", "TIGR00431.HMM", "TIGR00435.HMM",
                                  "TIGR00436.HMM", "TIGR00442.HMM", "TIGR00445.HMM",
                                  "TIGR00456.HMM", "TIGR00459.HMM", "TIGR00460.HMM",
                                  "TIGR00468.HMM", "TIGR00472.HMM", "TIGR00487.HMM",
                                  "TIGR00496.HMM", "TIGR00539.HMM", "TIGR00580.HMM",
                                  "TIGR00593.HMM", "TIGR00615.HMM", "TIGR00631.HMM",
                                  "TIGR00634.HMM", "TIGR00635.HMM", "TIGR00643.HMM",
                                  "TIGR00663.HMM", "TIGR00717.HMM", "TIGR00755.HMM",
                                  "TIGR00810.HMM", "TIGR00922.HMM", "TIGR00928.HMM",
                                  "TIGR00959.HMM", "TIGR00963.HMM", "TIGR00964.HMM",
                                  "TIGR00967.HMM", "TIGR01009.HMM", "TIGR01011.HMM",
                                  "TIGR01017.HMM", "TIGR01021.HMM", "TIGR01029.HMM",
                                  "TIGR01032.HMM", "TIGR01039.HMM", "TIGR01044.HMM",
                                  "TIGR01059.HMM", "TIGR01063.HMM", "TIGR01066.HMM",
                                  "TIGR01071.HMM", "TIGR01079.HMM", "TIGR01082.HMM",
                                  "TIGR01087.HMM", "TIGR01128.HMM", "TIGR01146.HMM",
                                  "TIGR01164.HMM", "TIGR01169.HMM", "TIGR01171.HMM",
                                  "TIGR01302.HMM", "TIGR01391.HMM", "TIGR01393.HMM",
                                  "TIGR01394.HMM", "TIGR01510.HMM", "TIGR01632.HMM",
                                  "TIGR01951.HMM", "TIGR01953.HMM", "TIGR02012.HMM",
                                  "TIGR02013.HMM", "TIGR02027.HMM", "TIGR02075.HMM",
                                  "TIGR02191.HMM", "TIGR02273.HMM", "TIGR02350.HMM",
                                  "TIGR02386.HMM", "TIGR02397.HMM", "TIGR02432.HMM",
                                  "TIGR02729.HMM", "TIGR03263.HMM", "TIGR03594.HMM",
                                  "TIGR03625.HMM", "TIGR03632.HMM", "TIGR03654.HMM",
                                  "TIGR03723.HMM", "TIGR03725.HMM", "TIGR03953.HMM"]}

    # New Version of AR53_MARKERS
    AR53_MARKERS = {"PFAM": ["PF04919.13.hmm", "PF07541.13.hmm", "PF01000.27.hmm",
                             "PF00687.22.hmm", "PF00466.21.hmm", "PF00827.18.hmm", "PF01280.21.hmm", "PF01090.20.hmm",
                             "PF01200.19.hmm", "PF01015.19.hmm", "PF00900.21.hmm", "PF00410.20.hmm"],
                    "TIGRFAM": ["TIGR00037.HMM", "TIGR00064.HMM", "TIGR00111.HMM",
                                "TIGR00134.HMM", "TIGR00279.HMM", "TIGR00291.HMM", "TIGR00323.HMM",
                                "TIGR00335.HMM", "TIGR00373.HMM", "TIGR00405.HMM", "TIGR00448.HMM",
                                "TIGR00483.HMM", "TIGR00491.HMM", "TIGR00522.HMM", "TIGR00967.HMM",
                                "TIGR00982.HMM", "TIGR01008.HMM", "TIGR01012.HMM", "TIGR01018.HMM",
                                "TIGR01020.HMM", "TIGR01028.HMM", "TIGR01046.HMM", "TIGR01052.HMM",
                                "TIGR01171.HMM", "TIGR01213.HMM", "TIGR01952.HMM", "TIGR02236.HMM",
                                "TIGR02338.HMM", "TIGR02389.HMM", "TIGR02390.HMM", "TIGR03626.HMM",
                                "TIGR03627.HMM", "TIGR03628.HMM", "TIGR03629.HMM", "TIGR03670.HMM",
                                "TIGR03671.HMM", "TIGR03672.HMM", "TIGR03673.HMM", "TIGR03674.HMM",
                                "TIGR03676.HMM", "TIGR03680.HMM"]}

    # Information for Multiple hits markers:
    DEFAULT_MULTIHIT_THRESHOLD = 10.0

    # Information for aligning genomes
    DEFAULT_DOMAIN_THRESHOLD = 10.0
    AR_MARKER_COUNT = 53
    BAC_MARKER_COUNT = 120

    # Information about alignment Fraction to resolve fastANI results
    AF_THRESHOLD = 0.5

    PPLACER_MIN_RAM_BAC_FULL = 320
    PPLACER_MIN_RAM_BAC_SPLIT = 55
    PPLACER_MIN_RAM_ARC = 40

    FASTANI_SPECIES_THRESHOLD = 95.0
    FASTANI_GENOMES_EXT = "_genomic.fna.gz"

    # Mash configuration
    MASH_SKETCH_FILE = 'gtdb_ref_sketch.msh'
    MASH_K_VALUE = 16
    MASH_S_VALUE = 5000
    MASH_MAX_DISTANCE = 0.15
    MASH_D_VALUE = MASH_MAX_DISTANCE
    MASH_V_VALUE = 1.0

    # Config values for checking GTDB-Tk on startup.
    GTDBTK_VER_CHECK = True
    GTDBTK_VER_TIMEOUT = 3  # seconds

    # Internal settings used for logging.
    LOG_TASK = 21

    # To avoid multiple hits of parsing files
    _generic_path = None
    _red_dist_bac_dict = None
    _red_dist_arc_dict = None
    _version_data = None

    @property
    def GENERIC_PATH(self):
        if self._generic_path is None:
            try:
                # expandvars is required to transform things like $HOME
                out = os.path.expandvars(os.environ['GTDBTK_DATA_PATH'])
                self._generic_path = out
            except KeyError:
                print('\n' + '=' * 80)
                print(' ERROR '.center(80))
                print('_' * 80 + '\n')
                print("The 'GTDBTK_DATA_PATH' environment variable is not defined.".center(80) + '\n')
                print('Please set this variable to your reference data package.'.center(80))
                print('https://ecogenomics.github.io/GTDBTk/installing/index.html'.center(80))
                print('=' * 80)
                sys.exit(1)
        return self._generic_path

    @property
    def MSA_FOLDER(self):
        return os.path.join(self.GENERIC_PATH, 'msa/')

    @property
    def MASK_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'masks/')

    @property
    def PPLACER_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'pplacer/')

    @property
    def FASTANI_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'fastani/')

    @property
    def MASH_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'mash/')

    @property
    def TAX_FOLDER(self):
        return os.path.join(self.GENERIC_PATH, 'taxonomy/')

    @property
    def RADII_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'radii/')

    @property
    def METADATA_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'metadata/')

    @property
    def RED_DIR(self):
        return os.path.join(self.GENERIC_PATH, "mrca_red/")

    @property
    def MARKER_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'markers/')

    @property
    def TIGRFAM_HMMS(self):
        return os.path.join(self.MARKER_DIR, 'tigrfam/tigrfam.hmm')

    @property
    def PFAM_HMM_DIR(self):
        return os.path.join(self.MARKER_DIR, 'pfam/')

    @property
    def SPLIT_DIR(self):
        return os.path.join(self.GENERIC_PATH, 'split')

    @property
    def BACKBONE_SPLIT_DIR(self):
        return os.path.join(self.SPLIT_DIR, 'backbone')

    @property
    def CLASS_LEVEL_SPLIT_DIR(self):
        return os.path.join(self.SPLIT_DIR, 'class_level')

    @property
    def BACKBONE_PPLACER_DIR(self):
        return os.path.join(self.BACKBONE_SPLIT_DIR, 'pplacer')

    @property
    def CLASS_LEVEL_PPLACER_DIR(self):
        return os.path.join(self.CLASS_LEVEL_SPLIT_DIR, 'pplacer')

    @property
    def BACKBONE_RED_DIR(self):
        return os.path.join(self.BACKBONE_SPLIT_DIR, 'red')

    @property
    def CLASS_LEVEL_RED_DIR(self):
        return os.path.join(self.CLASS_LEVEL_SPLIT_DIR, 'red')

    @property
    def CLASS_LEVEL_TREE_MAPPING_FILE(self):
        return os.path.join(self.CLASS_LEVEL_SPLIT_DIR, 'tree_mapping.tsv')

    @property
    def BACKBONE_RED_FILE(self):
        return os.path.join(self.BACKBONE_RED_DIR, 'backbone_red_value.tsv')

    @property
    def CLASS_LEVEL_RED_FILE(self):
        return os.path.join(self.CLASS_LEVEL_RED_DIR, 'red_value_{iter}.tsv')

    def _read_metadata_file(self):
        if self._red_dist_bac_dict is None or self._red_dist_arc_dict is None or self._version_data is None:
            try:
                with open(os.path.join(self.METADATA_DIR, "metadata.txt")) as metadataData:
                    for line in metadataData:
                        try:
                            line_infos = line.strip().split('=')
                            if line_infos[0] == 'RED_DIST_BAC_DICT':
                                self._red_dist_bac_dict = json.loads(line_infos[1])
                            elif line_infos[0] == 'RED_DIST_ARC_DICT':
                                self._red_dist_arc_dict = json.loads(line_infos[1])
                            elif line_infos[0] == 'VERSION_DATA':
                                self._version_data = line_infos[1]
                        except ValueError:
                            print(f"Skipping invalid line {repr(line)}")
            except IOError:
                print('\n' + '=' * 80)
                print(' ERROR '.center(80))
                print('_' * 80 + '\n')
                print('The GTDB-Tk reference data does not exist or is corrupted.'.center(80))
                print(('GTDBTK_DATA_PATH=%s' % self.GENERIC_PATH).center(80) + '\n')
                print('Please compare the checksum to those provided in the download repository.'.center(80))
                print('https://github.com/Ecogenomics/GTDBTk#gtdb-tk-reference-data'.center(80))
                print('=' * 80)
                sys.exit(1)
        return self._red_dist_bac_dict, self._red_dist_arc_dict, self._version_data

    @property
    def RED_DIST_BAC_DICT(self):
        return self._read_metadata_file()[0]

    @property
    def RED_DIST_ARC_DICT(self):
        return self._read_metadata_file()[1]

    @property
    def VERSION_DATA(self):
        return self._read_metadata_file()[2]

    """
    MSA file names
    """

    @property
    def CONCAT_BAC120(self):
        return os.path.join(self.MSA_FOLDER, f"gtdb_{self.VERSION_DATA}_bac120.faa")

    @property
    def CONCAT_AR53(self):
        return os.path.join(self.MSA_FOLDER, f"gtdb_{self.VERSION_DATA}_ar53.faa")

    @property
    def TAXONOMY_FILE(self):
        return os.path.join(self.TAX_FOLDER, "gtdb_taxonomy.tsv")

    @property
    def RADII_FILE(self):
        return os.path.join(self.RADII_DIR, "gtdb_radii.tsv")

    """
    Mask file names
    """

    @property
    def MASK_BAC120(self):
        return f"gtdb_{self.VERSION_DATA}_bac120.mask"

    @property
    def MASK_AR53(self):
        return f"gtdb_{self.VERSION_DATA}_ar53.mask"

    @property
    def MASK_RPS23(self):
        return f"gtdb_{self.VERSION_DATA}_rps23.mask"

    @property
    def PPLACER_BAC120_REF_PKG(self):
        return f"gtdb_{self.VERSION_DATA}_bac120.refpkg"

    @property
    def PPLACER_AR53_REF_PKG(self):
        return f"gtdb_{self.VERSION_DATA}_ar53.refpkg"

    @property
    def PPLACER_RPS23_REF_PKG(self):
        return f"gtdb_{self.VERSION_DATA}_rps23.refpkg"

    @property
    def FASTANI_GENOMES(self):
        return os.path.join(self.FASTANI_DIR, "database/")

    @property
    def FASTANI_GENOME_LIST(self):
        return os.path.join(self.FASTANI_DIR, "genome_paths.tsv")

    @property
    def MRCA_RED_BAC120(self):
        return os.path.join(self.RED_DIR, f"gtdbtk_{self.VERSION_DATA}_bac120.tsv")

    @property
    def MRCA_RED_AR53(self):
        return os.path.join(self.RED_DIR, f"gtdbtk_{self.VERSION_DATA}_ar53.tsv")

    def get_REF_HASHES(self,version=None):
        compatible_versions = [int(x.replace('r','')) for x in CONFIG.COMPATIBLE_REF_DATA_VERSIONS]
        if version is not None and version not in compatible_versions:
            raise ValueError(f"Version {version} is not compatible with this version of GTDB-Tk. Compatible versions are {compatible_versions}")

        if version is None or version==214:
            return {
                self.PPLACER_DIR: '6786e9fc16b31db7d6eaaa9f8cfa87a8a4974434',
                self.MASK_DIR: '8d5a2139feabbb70789c62155f3761d2aeed1601',
                self.MARKER_DIR: '163f542c3f0a40f59df45d453aa235b39aa96e27',
                self.RADII_DIR: '4753acc920001a1400788ee89cb4632900449055',
                self.MSA_FOLDER: '75df495678a121497e14346b453caf42f4b03922',
                self.METADATA_DIR: 'a089cc36bf79a40c7506019accc5f93e940d9fed',
                self.TAX_FOLDER: '89b12cf8106f326887599dcb30ef94ebba142035',
                self.FASTANI_DIR: 'e12824beccc15fe67a373e2aa8eee72feecf89c6',
                self.RED_DIR: 'c24a2f48bb0c1df38f92a8f526aa846f596c94c6'
            }
        elif version==207:
            return {
                self.PPLACER_DIR: '20903925a856a58b102a7b0ce160c5cbd2cf675b',
                self.MASK_DIR: '50e414a9de18170e8cb97f990f89ff60a0fe29d5',
                self.MARKER_DIR: '163f542c3f0a40f59df45d453aa235b39aa96e27',
                self.RADII_DIR: '8fd13b1c5d7a7b073ba96fb628581613b293a374',
                self.MSA_FOLDER: '24f250d7cf0eb0bc65dccd2f3c9247e553ea322f',
                self.METADATA_DIR: '9772fbeac1311b31e10293fa610eb33aa1ec8e15',
                self.TAX_FOLDER: '6fb0233b05633242369b40c026fd1ee53e266afa',
                self.FASTANI_DIR: '973c456c02f55bb82908a6811c7076e207e9b206',
                self.RED_DIR: '7b8b67b3157204b470c9eb809d3c39c4effffabc'
            }

    REF_HASHES = property(get_REF_HASHES)


# Export the class for import by other modules
@lru_cache(maxsize=1)
def __get_config():
    return __GTDBTkCommonConfig()


CONFIG = __get_config()



from os.path import join

# Command: identify
DIR_IDENTIFY = 'identify'
DIR_IDENTIFY_INTERMEDIATE = join(DIR_IDENTIFY, 'intermediate_results')
DIR_MARKER_GENE = join(DIR_IDENTIFY_INTERMEDIATE, 'marker_genes')
DIR_IDENTIFY_FASTA = join(DIR_IDENTIFY_INTERMEDIATE, 'single_copy_fasta')
PATH_BAC120_MARKER_SUMMARY = join(DIR_IDENTIFY, '{prefix}.bac120.markers_summary.tsv')
PATH_AR53_MARKER_SUMMARY = join(DIR_IDENTIFY, '{prefix}.ar53.markers_summary.tsv')
PATH_TLN_TABLE_SUMMARY = join(DIR_IDENTIFY, '{prefix}.translation_table_summary.tsv')
PATH_FAILS = join(DIR_IDENTIFY,'{prefix}.failed_genomes.tsv')

# Command: identify -> marker genes
GENOME_FILE_SUFFIX = "_genomic.fna"
PROTEIN_FILE_SUFFIX = "_protein.faa"
NT_GENE_FILE_SUFFIX = "_protein.fna"
GFF_FILE_SUFFIX = "_protein.gff"
TRANSLATION_TABLE_SUFFIX = "_translation_table.tsv"
CHECKSUM_SUFFIX = ".sha256"
TIGRFAM_SUFFIX = "_tigrfam.tsv"
TIGRFAM_TOP_HIT_SUFFIX = "_tigrfam_tophit.tsv"
PFAM_SUFFIX = "_pfam.tsv"
PFAM_TOP_HIT_SUFFIX = "_pfam_tophit.tsv"

# Command: align
DIR_ALIGN = 'align'
DIR_ALIGN_INTERMEDIATE = join(DIR_ALIGN, 'intermediate_results')
PATH_BAC120_FILTERED_GENOMES = join(DIR_ALIGN, '{prefix}.bac120.filtered.tsv')
PATH_AR53_FILTERED_GENOMES = join(DIR_ALIGN, '{prefix}.ar53.filtered.tsv')
PATH_BAC120_MSA = join(DIR_ALIGN, '{prefix}.bac120.msa.fasta')
PATH_AR53_MSA = join(DIR_ALIGN, '{prefix}.ar53.msa.fasta')
PATH_BAC120_USER_MSA = join(DIR_ALIGN, '{prefix}.bac120.user_msa.fasta')
PATH_AR53_USER_MSA = join(DIR_ALIGN, '{prefix}.ar53.user_msa.fasta')
PATH_FAILED_ALIGN_GENOMES = join(DIR_ALIGN_INTERMEDIATE, '{prefix}.align.failed.tsv')
PATH_FAILED_IDENTIFY_GENOMES = join(DIR_ALIGN_INTERMEDIATE, '{prefix}.identify.failed.tsv')
PATH_BAC120_MARKER_INFO = join(DIR_ALIGN_INTERMEDIATE, '{prefix}.bac120.marker_info.tsv')
PATH_AR53_MARKER_INFO = join(DIR_ALIGN_INTERMEDIATE, '{prefix}.ar53.marker_info.tsv')
DIR_ALIGN_MARKERS = join(DIR_ALIGN_INTERMEDIATE, 'markers')

# Command: classify
DIR_CLASSIFY = 'classify'
PATH_BAC120_TREE_FILE = join(DIR_CLASSIFY, '{prefix}.bac120.classify.tree')
PATH_AR53_TREE_FILE = join(DIR_CLASSIFY, '{prefix}.ar53.classify.tree')
PATH_BAC120_SUMMARY_OUT = join(DIR_CLASSIFY, '{prefix}.bac120.summary.tsv')
PATH_AR53_SUMMARY_OUT = join(DIR_CLASSIFY, '{prefix}.ar53.summary.tsv')
PATH_BACKBONE_BAC120_TREE_FILE = join(DIR_CLASSIFY, '{prefix}.backbone.bac120.classify.tree')
PATH_CLASS_LEVEL_BAC120_TREE_FILE = join(DIR_CLASSIFY, '{prefix}.bac120.classify.tree.{iter}.tree')
PATH_BAC120_CONFLICT = join(DIR_CLASSIFY, '{prefix}.bac120.conflict.tsv')
PATH_AR53_DISAPPEARING_GENOMES = join(DIR_CLASSIFY, '{prefix}.ar53.disappearing_genomes.tsv')
PATH_BAC120_DISAPPEARING_GENOMES = join(DIR_CLASSIFY, '{prefix}.bac120.disappearing_genomes.tsv')

DIR_CLASSIFY_INTERMEDIATE = join(DIR_CLASSIFY, 'intermediate_results')
PATH_BAC120_TREE_MAPPING = join(DIR_CLASSIFY, '{prefix}.bac120.tree.mapping.tsv')
PATH_BAC120_RED_DICT = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.bac120.red_dictionary.tsv')
PATH_AR53_RED_DICT = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.ar53.red_dictionary.tsv')
PATH_BAC120_PPLACER_CLASS = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.bac120.classification_pplacer.tsv')
PATH_AR53_PPLACER_CLASS = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.ar53.classification_pplacer.tsv')
PATH_BAC120_BACKBONE_PPLACER_CLASS = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.bac120.backbone.classification_pplacer.tsv')
PATH_BAC120_CLASS_LEVEL_PPLACER_CLASS = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.bac120.class_level.classification_pplacer_tree_{iter}.tsv')
PATH_BAC120_PRESCREEN_MSA = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.bac120.prescreened.msa.fasta')
PATH_AR53_PRESCREEN_MSA = join(DIR_CLASSIFY_INTERMEDIATE, '{prefix}.ar53.prescreened.msa.fasta')

#Command: ani_screen
DIR_ANISCREEN = join(DIR_CLASSIFY, 'ani_screen')
DIR_ANISCREEN_INTERMEDIATE = join(DIR_ANISCREEN, 'intermediate_results')
PATH_BAC120_ANI_FASTANI = join(DIR_ANISCREEN, '{prefix}.bac120_fastani.summary.tsv')
PATH_AR53_ANI_FASTANI = join(DIR_ANISCREEN, '{prefix}.ar53_fastani.summary.tsv')


DIR_PPLACER = join(DIR_CLASSIFY_INTERMEDIATE, 'pplacer')
PATH_BAC120_PPLACER_OUT = join(DIR_PPLACER, 'pplacer.bac120.out')
PATH_AR53_PPLACER_OUT = join(DIR_PPLACER, 'pplacer.ar53.out')
PATH_BAC120_PPLACER_JSON = join(DIR_PPLACER, 'pplacer.bac120.json')
PATH_AR53_PPLACER_JSON = join(DIR_PPLACER, 'pplacer.ar53.json')

# SPLIT TREE
PATH_BACKBONE_BAC120_PPLACER_OUT = join(DIR_PPLACER, 'pplacer.backbone.bac120.out')
PATH_BACKBONE_BAC120_PPLACER_JSON = join(DIR_PPLACER, 'pplacer.backbone.bac120.json')
DIR_CLASS_LEVEL_PPLACER = join(DIR_PPLACER, 'tree_{iter}')
PATH_CLASS_LEVEL_BAC120_SUBMSA = join(DIR_CLASS_LEVEL_PPLACER, 'user_msa_file.fasta')
PATH_CLASS_LEVEL_BAC120_PPLACER_OUT = join(DIR_CLASS_LEVEL_PPLACER, 'pplacer.class_level.bac120.out')
PATH_CLASS_LEVEL_BAC120_PPLACER_JSON = join(DIR_CLASS_LEVEL_PPLACER, 'pplacer.class_level.bac120.json')

# Command: infer
DIR_INFER = 'infer'
DIR_INFER_INTERMEDIATE = join(DIR_INFER, 'intermediate_results')
PATH_MARKER_UNROOTED_TREE = join(DIR_INFER_INTERMEDIATE, '{prefix}.{marker}.unrooted.tree')
PATH_UNROOTED_TREE = join(DIR_INFER_INTERMEDIATE, '{prefix}.unrooted.tree')
PATH_MARKER_TREE_LOG = join(DIR_INFER_INTERMEDIATE, '{prefix}.{marker}.tree.log')
PATH_TREE_LOG = join(DIR_INFER_INTERMEDIATE, '{prefix}.tree.log')
PATH_MARKER_FASTTREE_LOG = join(DIR_INFER_INTERMEDIATE, '{prefix}.{marker}.fasttree.log')
PATH_FASTTREE_LOG = join(DIR_INFER_INTERMEDIATE, '{prefix}.fasttree.log')

PATH_BAC120_UNROOTED_TREE = join(DIR_INFER_INTERMEDIATE, '{prefix}.bac120.unrooted.tree')
PATH_AR53_UNROOTED_TREE = join(DIR_INFER_INTERMEDIATE, '{prefix}.ar53.unrooted.tree')
PATH_BAC120_ROOTED_TREE = join(DIR_INFER_INTERMEDIATE, '{prefix}.bac120.rooted.tree')
PATH_AR53_ROOTED_TREE = join(DIR_INFER_INTERMEDIATE, '{prefix}.ar53.rooted.tree')
PATH_BAC120_DECORATED_TREE = join(DIR_INFER, '{prefix}.bac120.decorated.tree')
PATH_AR53_DECORATED_TREE = join(DIR_INFER, '{prefix}.ar53.decorated.tree')

# Command: ani_rep
DIR_ANI_REP_INT = join('intermediate_results')
DIR_ANI_REP_INT_MASH = join(DIR_ANI_REP_INT, 'mash')

# General files
PATH_WARNINGS = '{prefix}.warnings.log'